<?php
// vim: set ai ts=4 sw=4 ft=php:
namespace FreePBX\modules;

class Bria implements \BMO {

	public function __construct($freepbx = null) {
		if ($freepbx == null) {
			throw new \Exception("Not given a FreePBX Object");
		}
		$this->freepbx = $freepbx;
		$this->db = $freepbx->Database;
		$this->userman = $freepbx->Userman;

		//Ensure we have a db table before we try to initialize variables
		if ($this->checkIfTableExists() === true) {
			$this->ccsUsername = $this->getSetting('username');
			$this->ccsPassword = $this->getSetting('password');


			if (!class_exists('PestBria')) {
				require_once(dirname(__FILE__) . '/functions.inc/PestBria.php');
			}

			if (!empty($this->ccsUsername) && !empty($this->ccsPassword)) {
				$this->pest = new \PestBria($this->ccsUsername, $this->ccsPassword);
			}
		}
	}

	public function doConfigPageInit($page) {}
	public function install() {}
	public function uninstall() {}
	public function backup(){}
	public function restore($backup){}

	public function getActionBar($request) {
		$buttons = array();
		switch($request['display']) {
			case 'bria':
				$buttons = array(
					'reset' => array(
						'name' => 'reset',
						'id' => 'reset',
						'value' => _('Reset'),
						'hidden' => ''
					),
					'submit' => array(
						'name' => 'submit',
						'id' => 'submit',
						'value' => _('Submit'),
						'hidden' => ''
					)
				);
			break;
		}
		return $buttons;
	}

	public function usermanShowPage() {
		if(isset($_REQUEST['action'])) {
			$ipaddress = $this->getSetting('ipaddr');
			$groupname = $this->getSetting('groupname');
			$enabled = false;

			if (empty($this->ccsUsername) && empty($this->ccsPassword) && empty($ipaddress) && empty($groupname)) {
				return array();
			}

			if(isset($_REQUEST['action'])) {
				switch($_REQUEST['action']) {
					case 'showgroup':
						$enabled = $this->userman->getModuleSettingByGID($_REQUEST['group'], 'bria', 'enabled');
						return array(
							array(
								"title" => sprintf(_("%s Cloud Solutions"),'Bria'),
								"rawname" => "bria",
								"content" => load_view(__DIR__.'/views/userman_hook.php',array("mode" => "group", "enabled" => $enabled))
							)
						);
					break;
					case 'showuser':
						$enabled = $this->userman->getModuleSettingByID($_REQUEST['user'], 'bria', 'enabled', true);
						$transport = \FreePBX::Userman()->getModuleSettingByID($_REQUEST['user'], 'bria', 'transport');
						switch(strtolower($transport)) {
							case 'tcp':
								$transport = 'tcp';
								break;
							case 'udp':
							default:
								$transport = 'udp';
								break;
						}
						return array(
							array(
								"title" => sprintf(_("%s Cloud Solutions"),'Bria'),
								"rawname" => "bria",
								"content" => load_view(__DIR__.'/views/userman_hook.php',array("mode" => "user", "enabled" => $enabled, "transport" => $transport))
							)
						);
					break;
					case 'addgroup':
					case 'adduser':
						$mode = ($_REQUEST['action'] == 'addgroup') ? 'group' : 'user';
						return array(
							array(
								"title" => sprintf(_("%s Cloud Solutions"),'Bria'),
								"rawname" => "bria",
								"content" => load_view(__DIR__.'/views/userman_hook.php',array("mode" => $mode, "enabled" => true))
							)
						);
					break;
				}
			}
		}
	}

	public function usermanDelGroup($id,$display,$data) {
	}

	public function usermanAddGroup($id, $display, $data) {
		$this->usermanUpdateGroup($id,$display,$data);
	}

	public function usermanUpdateGroup($id,$display,$data) {
		if($display == 'userman' && isset($_POST['type']) && $_POST['type'] == 'group') {
			if(isset($_POST['bria|enable']) && $_POST['bria|enable'] == 'true') {
				$this->userman->setModuleSettingByGID($id,'bria','enabled', true);
			} else {
				$this->userman->setModuleSettingByGID($id,'bria','enabled', false);
			}
		}
		$group = $this->userman->getGroupByGID($id);
		foreach($group['users'] as $user) {
			$enabled = $this->userman->getCombinedModuleSettingByID($user, 'bria', 'enabled');
			$userData = $this->userman->getUserByID($user);
			if ($enabled) {
				//We need to save the password for this user in plain text
				$password = $this->userman->getModuleSettingByID($user, 'bria', 'password');
				$this->updateBriaUser($userData['username'], $userData['username'], $password);

				$this->createDevice($userData['username'], $userData['username'], $user, $userData['default_extension']);
			} else {
				$this->deleteDevice($userData['username'], $userData['default_extension']);
			}
		}
	}

	public function usermanAddUser($id, $display, $data) {
		$this->usermanUpdateUser($id, $display, $data);
	}

	public function usermanUpdateUser($id, $display, $data) {
		//determine if this was done through ucp or not
		if ($display == 'userman' && isset($_POST['type']) && $_POST['type'] == 'user') {
			if($_REQUEST['bria|enable'] == 'true') {
				$this->userman->setModuleSettingByID($id, 'bria', 'enabled', true);
			} elseif($_REQUEST['bria|enable'] == 'false') {
				$this->userman->setModuleSettingByID($id, 'bria', 'enabled', false);
			} else {
				$this->userman->setModuleSettingByID($id, 'bria', 'enabled', null);
			}
			$transport = ($_REQUEST['bria|transport'] == 'tcp') ? 'tcp' : 'udp';
			$this->userman->setModuleSettingByID($id, 'bria', 'transport', $transport);
		}
		$enabled = $this->userman->getCombinedModuleSettingByID($id, 'bria', 'enabled');

		$usermanUser = $this->userman->getUserByID($id);
		if ($enabled && $usermanUser['default_extension'] != "none") {
			$data['prevUsername'] = !empty($data['prevUsername']) ? $data['prevUsername'] : '';
			//We need to save the password for this user in plain text
			if(!empty($data['password'])) {
				$this->userman->setModuleSettingByID($id, 'bria', 'password', $data['password']);
			}

			if(!empty($data['password']) || ($data['username'] != $data['prevUsername'])) {
				$this->updateBriaUser($data['prevUsername'], $data['username'], $data['password']);
			}

			$this->createDevice($data['username'], $data['prevUsername'], $id, $usermanUser['default_extension']);
		} elseif($usermanUser['default_extension'] != "none" && !empty($data['prevUsername'])) {
			$this->deleteDevice($data['prevUsername'], $usermanUser['default_extension']);
		}

		//If this is from userman a reload is needed as we are an admin and we need to check the device
		if ($display == 'userman') {
			needreload();
		}
	}

	public function usermanDelUser($id, $display, $data) {
		//Delete the device associated with the user
		$usermanUser = $this->userman->getUserByID($id);
		if ($usermanUser['default_extension'] != "none") {
			$this->deleteDevice($usermanUser['username'], $usermanUser['default_extension']);
		}

		needreload();
	}

	function getAllSettings() {
		$sql = 'SELECT * FROM bria_settings';

		$sth = $this->db->prepare($sql);
		$sth->execute();

		return $sth->fetchAll(\PDO::FETCH_ASSOC);
	}

	function getSetting($setting) {
		$sql = 'SELECT value FROM bria_settings WHERE `key` = :setting';

		$sth = $this->db->prepare($sql);
		$sth->execute(array(":setting" => $setting));
		$results = $sth->fetch(\PDO::FETCH_ASSOC);

		$results = (!empty($results['value'])) ? $results['value'] : null;

		return $results;
	}

	function checkIfTableExists() {
		$sql = "SHOW TABLES LIKE 'bria_settings'";

		$sth = $this->db->prepare($sql);
		$sth->execute();
		$results = $sth->fetch(\PDO::FETCH_ASSOC);

		return (!empty($results)) ? true : false;
	}

	function setSetting($setting, $value) {
		$sql = 'REPLACE INTO bria_settings (`key`, `value`) VALUES (:setting, :value)';
		$sth = $this->db->prepare($sql);
		$ret = $sth->execute(array(":setting" => $setting, ":value" => $value));

		return $ret;
	}

	function setMultipleSettings($settings = array()) {
		if (!empty($settings)) {
			foreach ($settings as $key => $value) {
				$sql = 'REPLACE INTO bria_settings (`key`, `value`) VALUES (:setting, :value)';
				$sth = $this->db->prepare($sql);
				$ret = $sth->execute(array(":setting" => $key, ":value" => $value));

				if (!$ret) {
					return false;
				}
			}
			return true;
		}
		return false;
	}

	function getPrefix() {
		$prefix = $this->getSetting('prefix');

		$prefix = !empty($prefix) ? $prefix : '999';

		return $prefix;
	}

	function updatePrefix($oldPrefix) {
		$users = $this->getAllUsers();
		//Run through each enabled user deleting and recreating the devices needed for lync
		foreach ($users as $key => $user) {
			$this->deleteDevice($user['username'], $user['default_extension'], $oldPrefix);
			$this->createDevice($user['username'], null, $user['uid'], $user['default_extension']);
		}

		//Add the apply changes to FreePBX
		needreload();
	}

	//This is used when making admin setting changes which is why we don't require a reload
	function updateBriaUserSettings() {
		$users = $this->getAllUsers();
		//Run through each enabled user deleting and recreating the devices needed for lync
		foreach ($users as $key => $user) {
			$this->deleteDevice($user['username'], $user['default_extension']);
			$this->createDevice($user['username'], null, $user['uid'], $user['default_extension']);
		}
	}

	function getUserByExtension($extension) {
		$sql = "SELECT u.id, u.username, u.default_extension, u.fname, u.lname
		FROM userman_users u
		JOIN userman_users_settings s ON s.uid = u.id
		WHERE s.module = 'bria'
		AND s.key = 'enabled'
		AND s.val = 1
		AND u.default_extension = :extension";

		$sth = $this->db->prepare($sql);
		$sth->execute(array(":extension" => $extension));
		return $sth->fetch(\PDO::FETCH_ASSOC);
	}

	function getAllUsers() {
		$sql = "SELECT u.id, u.username, u.default_extension, u.fname, u.lname
		FROM userman_users u
		JOIN userman_users_settings s ON s.uid = u.id
		WHERE s.module = 'bria'
		AND s.key = 'enabled'
		AND s.val = 1";

		$sth = $this->db->prepare($sql);
		$sth->execute();
		return $sth->fetchAll(\PDO::FETCH_ASSOC);
	}

	function deleteDevice($username, $extension, $prefix = null) {
		if (!isset($prefix)) {
			$prefix = $this->getPrefix();
		}

		$ccsGroupName = $this->getSetting('groupname');

		//if no ccs username and password exit
		if (empty($this->ccsUsername) || empty($this->ccsPassword) || empty($ccsGroupName)) {
			return false;
		}

		$this->freepbx->Core->delDevice($prefix.$extension);

		//Delete Bria device on the server
		try {
			$getResponse = $this->pest->getUser(array(
				'userName' => $username . '@' . $ccsGroupName,
			));
		} catch (Exception $e) {
			return false;
		}

		//Delete a User if they already exist
		if (is_object($getResponse) && isset($getResponse->CcsUser) && $getResponse->CcsUser->attributes()->userName) {
			try {
				$delResponse = $this->pest->deleteUser(array(
					'userName' => $getResponse->CcsUser->attributes()->userName,
				));
			} catch (Exception $e) {
				return false;
			}

			if ($this->pest->lastStatus() === 200) {
				return true;
			}
		}

		return false;
	}

	function createDevice($username, $prevUsername, $uid, $extension) {
		$prefix = $this->getPrefix();
		$ccsIpAddress = $this->getSetting('ipaddr');
		$ccsGroupName = $this->getSetting('groupname');

		//if no ccs username and password exit
		if (empty($this->ccsUsername) || empty($this->ccsPassword) || empty($ccsGroupName) || empty($ccsIpAddress)) {
			return false;
		}

		if ($extension == 'none') {
			return false;
		}

		$usr = $this->freepbx->Core->getUser($extension);
		$dev = $this->freepbx->Core->getDevice($prefix.$extension);

		//get the original extensions information
		$extensionDev = $this->freepbx->Core->getDevice($extension);

		if (empty($usr)) {
			return false;
		}

		if (empty($dev)) {
			$userTransport = \FreePBX::Userman()->getModuleSettingByID($_REQUEST['user'], 'bria', 'transport');

			$settings = $this->freepbx->Core->generateDefaultDeviceSettings('sip',$prefix.$extension,$usr['name'].' Bria Client');
			if(!empty($extensionDev)) {
				foreach($extensionDev as $key => $value) {
					if(isset($settings[$key]['value'])) {
						$settings[$key]['value'] = $value;
					}
				}
			}
			$settings['devicetype']['value'] = 'fixed';
			$settings['user']['value'] = $extension;
			$settings['secret']['value'] = $extensionDev['secret'];
			$settings['transport']['value'] = $userTransport;
			$this->freepbx->Core->addDevice($prefix.$extension,'sip',$settings);

			foreach ($settings as $key => $value) {
				if (!empty($res['devinfo_'.$key])) {
					$_REQUEST['devinfo_'.$key] = $res['devinfo_'.$key];
				}
			}

		}

		//Create Bria device on the server
		//Determine if the user already exists
		try {
			$getResponse = $this->pest->getUser(array(
			'userName' => $prevUsername . '@' . $ccsGroupName,
			));
		} catch (Exception $e) {
			return false;
		}

		//If user already exists, delete them so that we can recreate them
		if ($this->pest->lastStatus() === 200 && isset($getResponse->CcsUser) && isset($getResponse->CcsUser->attributes()->userName)) {
			$delResponse = $this->pest->deleteUser(array(
			'userName' => $getResponse->CcsUser->attributes()->userName,
			));

			if ($this->pest->lastStatus() !== 200) {
				return false;
			}
		}

		//Get the original extension information for the sip secret
		$dev = $this->freepbx->Core->getDevice($extension);

		//Get the user password from userman
		$userPassword = \FreePBX::Userman()->getModuleSettingByID($_REQUEST['user'], 'bria', 'password');

		//Add Bria user
		$user = array(
			'userName' => $username  . '@' . $ccsGroupName,
			'password' => $userPassword, //Grab the plain text password to use
			'profileName' => 'sip.only',
		);

		$attributes = array(
			'account1Sip.credentials.authorizationName' => $prefix.$extension,
			'account1Sip.accountName' => $usr['name'],
			'account1Sip.credentials.displayName' => $usr['name'],
			'account1Sip.credentials.password' => $dev['secret'],
			'account1Sip.credentials.username' => $prefix.$extension,
			'account1Sip.domain' => $ccsIpAddress,
		);

		//Get the voicemail feature code from feature codes and set it if active
		$fcc = new \featurecode('voicemail', 'myvoicemail');
		$voicemailFeatureCodeActive = $fcc->isEnabled();
		if ($voicemailFeatureCodeActive) {
			$voicemailFeatureCode = $fcc->getCode();
			$attributes['account1Sip.voicemailNumber'] = $voicemailFeatureCode;
		}

		if (function_exists('xmpp_users_get')) {

			$xmppUser = xmpp_users_get($uid);

			if (!empty($xmppUser['username']) && !empty($xmppUser['password'])) {

				$xmppDomain = xmpp_opts_get('domain');

				$user['profileName'] = 'sip.xmpp';

				$attributes['account2Xmpp.credentials.displayName'] = $usr['name'];
				$attributes['account2Xmpp.credentials.username'] = $xmppUser['username'];
				$attributes['account2Xmpp.credentials.password'] = $xmppUser['password'];
				$attributes['account2Xmpp.domain'] = $xmppDomain;
			}
		} else if(\FreePBX::Modules()->moduleHasMethod('xmpp',"getUser")) { //we have a newer xmpp

			$xmppUser = \FreePBX::Xmpp()->getUserByID($uid);

			if (!empty($xmppUser['username']) && !empty($xmppUser['password'])) {

				$user['profileName'] = 'sip.xmpp';

				$attributes['account2Xmpp.credentials.displayName'] = $usr['name'];
				$attributes['account2Xmpp.credentials.username'] = $xmppUser['username'];
				$attributes['account2Xmpp.credentials.password'] = $xmppUser['password'];
				$attributes['account2Xmpp.domain'] = $xmppUser['domain'];
			}
		}

	if ($this->getSetting('web_browser_enable')) {
			$attributes['desk.feature.webBrowser.enabled'] = true;
			$attributes['desk.feature.webBrowser.name'] = $this->getSetting('web_browser_name');
			$attributes['desk.feature.webBrowser.url'] = $this->getSetting('web_browser_url');
		}

		$attributes['account.notification.administratorName'] = 'FreePBX';

		$user = array_merge($user, $attributes);

		try {
			$addResponse =  $this->pest->addUser($user);
		} catch (Exception $e) {
			return false;
		}

		if ($this->pest->lastStatus() === 200) {
			return true;
		}

		return false;
	}

	function updateBriaUser($prevusername, $username, $password) {
		$ccsGroupName = $this->getSetting('groupname');

		if (empty($paramsToUpdate)) {
			return false;
		}

		//Create Bria device on the server
		//Determine if the user already exists
		try {
			$getResponse = $this->pest->getUser(array(
				'userName' => $prevusername . '@' . $ccsGroupName,
			));
		} catch (Exception $e) {
			return false;
		}

		//If user already exists, delete them so that we can recreate them
		if ($this->pest->lastStatus() === 200 && isset($getResponse->CcsUser) && isset($getResponse->CcsUser->attributes()->userName)) {
			try{
				$user = array(
					'userName' => $prevusername  . '@' . $ccsGroupName,
					'password' => $password,
					'newUserName' => $username
				);
				$this->pest->editUser($user);
			} catch (Exception $e) {
					return false;
			}

			if ($this->pest->lastStatus() !== 200) {
				return false;
			}

			return true;
		}
	}

	public function dashboardIgnoreExt(){
		$prefix = $this->getPrefix();
		return array(array('length' => strlen($prefix), 'value' => $prefix));
	}
}
